/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/
/*
 * This header file contains the definitions for resources.
 *
 * Other required header files:
 *	"portability.h"
 *	"attribute.h"
 *	"list_link.h"
 *
 * Resources are "a special case" of attributes.  Resources use similiar
 * structures as attributes.  Certain types, type related functions,
 * and flags may differ between the two.
 *
 * Within the resource structure, the value is contained in an attribute
 * substructure, this is done so the various attribute decode and encode
 * routines can be "reused".
 *
 * For any server, queue or job attribute which is a set of resources,
 * the attribute points to an list  of "resource" structures.
 * The value of the resource is contained in these structures.
 *
 * Unlike "attributes" which are typically identical between servers
 * within an administrative domain,  resources vary between systems.
 * Hence, the resource instance has a pointer to the resource definition
 * rather than depending on a predefined index.
 */

typedef struct resource {
	list_link 	     rs_link;	/* link to other resources in list */
	struct resource_def *rs_defin;	/* pointer to definition entry */
	attribute	     rs_value;	/* attribute struct holding value */
} resource;
	
typedef struct resource_def {
	char   *rs_name;
	int   (*rs_decode) A_((attribute *prsc,char *name,char *rn,char *val));
	int   (*rs_encode) A_((attribute *prsv, list_head *phead, char *atname,
			       char *rsname, int mode));
	int   (*rs_set) A_((attribute *old, attribute *new, enum batch_op op));
	int   (*rs_comp) A_((attribute *prsc, attribute *with));
	void  (*rs_free) A_((attribute *prsc));
	int   (*rs_action) A_((resource *presc, attribute *pat, int actmode));
	int	rs_flags:ATRFLAG;	/* flags: R/O, ..., see attribute.h */
	int	rs_type:ATRTYPE;	/* type of resource,see attribute.h */
} resource_def;

extern resource_def svr_resc_def[];	/* the resource definition array */
extern int	    svr_resc_size;	/* size (num elements) in above  */

extern resource     *add_resource_entry A_((attribute *, resource_def *));
extern resource_def *find_resc_def A_((resource_def *, char *, int));
extern resource     *find_resc_entry A_((attribute *, resource_def *));
